`include "rtavr_defs.v"
// global defines :  SOCTOP_INCLUDE_IOR, SOCTOP_INCLDUE_RAM, SOCTOP_INCLUDE_ROM

//`define SOCTOP_INCLUDE_IOR

//
// Module Name: soctop
//   ver 002
// 
// Target Device:   xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal:     Balanced
//
//                                        all-in-one  
// Number of Slice Flip Flops:              247
// Number of 4 input LUTs:                  722
// Number of occupied Slices:               472
// Total Number of 4 input LUTs:            762
// Number of bonded IOBs :                   26
//   IOB Flip Flops :                        24
// Number of BUFGMUXs :                       2
// Number used for Dual Port RAMs            16
// Number of RAMB16BWEs :                     2
// Clock to Setup 
//on destination clock CLK2X              13.030

module soctop (
		input RESET
		, input CLK
`ifdef SOCTOP_INCLUDE_IOR
		// external I/O
  `ifdef IOR_HAVE_PORTA
		, inout [7:0] PORTA
  `endif
  `ifdef IOR_HAVE_PORTC
		, inout [7:0] PORTB
  `endif
  `ifdef IOR_HAVE_PORTC
		, inout [7:0] PORTC
  `endif
`else
		, output IOR_WEB, IOR_OE
		, output [5:0] IOR_ADDRB
		, output [7:0] IOR_DIB
		, input [7:0] IOR_DOB

		// SREG(FLAGS) & SP
		, input [7:0]; SREG_IN
		, output [7:0]; SREG_OUT
		, input [15:0]; SP_IN
		, output [15:0]; SP_OUT
`endif
		);

`ifdef SOCTOP_INCLUDE_IOR
  wire IOR_WEB, IOR_OE;
  wire [5:0] IOR_ADDRB;
  wire [7:0] IOR_DIB;
  wire [7:0] IOR_DOB;

  wire [7:0] SREG_IN;
  wire [7:0] SREG_OUT;
  wire [15:0] SP_IN;
  wire [15:0] SP_OUT;
  
  wire IOR_RST_INT0 = 0;
  wire IOR_RST_TIM0_OVF = 0;
  wire IOR_RST_TIM0_COMPA = 0;
  wire IOR_RST_TIM0_COMPB = 0;

  rtavr_ior IOR (.CLK(CLK), .RESET(RESET), .WR(IOR_WEB), .RD(IOR_OE)
		, .ADDR(IOR_ADDRB[5:0]), .DI(IOR_DIB[7:0]), .DO(IOR_DOB[7:0])
  `ifdef IOR_HAVE_PORTA
		, .PORTA(PORTA[7:0])
  `endif
  `ifdef IOR_HAVE_PORTB
		, .PORTB(PORTB[7:0])
  `endif
  `ifdef IOR_HAVE_PORTC
		, .PORTC(PORTC[7:0])
  `endif
  `ifdef IOR_HAVE_SREG_SP
		// SREG(FLAGS) & SP
		, .SREG_IN(SREG_OUT[7:0]) , .SREG_OUT(SREG_IN[7:0])
		, .SP_IN(SP_OUT[15:0]) , .SP_OUT(SP_IN[15:0])
  `endif
		// INTERRUPTS 
		, .RST_INT0(IOR_RST_INT0)
		, .INT0(IOR_INT0)
		, .RST_TIM0_OVF(IOR_RST_TIM0_OVF)
		, .TIM0_OVF(IOR_TIM0_OVF)
		, .RST_TIM0_COMPA(IOR_RST_TIM0_COMPA)
		, .TIM0_COMPA(IOR_TIM0_COMPA)
		, .RST_TIM0_COMPB(IOR_RST_TIM0_COMPB)
		, .TIM0_COMPB(IOR_TIM0_COMPB)
		);
`endif

  wire RAM_WEB;
  wire RAM_OE = 1;
  wire [10:0] RAM_ADDRB;
  wire [7:0] RAM_DIB;
  wire [7:0] RAM_DOB;

  wire RAM_WEA = 0;
  wire [10:0] RAM_ADDRA = 0;
  wire [7:0] RAM_DIA = 0;
 
  rtavr_sram_2KB RAM(.CLK(CLK), .RESET(RESET), .WEB(RAM_WEB), .OE(RAM_OE)
		, .ADDRB(RAM_ADDRB[10:0])
		, .DIB(RAM_DIB[7:0]), .DOB(RAM_DOB[7:0])
		, .WEA(RAM_WEA), .DIA(RAM_DIA[7:0]), .ADDRA(RAM_ADDRA[10:0])
		);

`ifdef SOCTOP_INCLUDE_ROM
  wire ROM_WEB, ROM_OE;
  wire [12:0] ROM_ADDRB;
  wire [7:0] ROM_DIB;
  wire [7:0] ROM_DOB;
  
  wire [11:0] ROM_ADDRA;
  wire [15:0] ROM_DOA;
  rtavr_rom #(.SIZE(`ROM_SIZE))
	ROM (.CLK(CLK), .RESET(RESET), .WEB(ROM_WEB), .OE(ROM_OE)
		, .ADDRB(ROM_ADDRB[12:0])
		, .DIB(ROM_DIB[7:0]), .DOB(ROM_DOB[7:0])
		, .ADDRA(ROM_ADDRA[11:0]), .DOA(ROM_DOA[15:0])
		);
`endif


  navre_core #(
	.pmem_width(11),
	.dmem_width(13) )
     CORE (
	.clk(CLK) , .rst(RESET)
	// input clk,
	// input rst,

	, .pmem_ce(ROM_OE)
	, .pmem_a(ROM_ADDRA[11:0])
	, .pmem_d(ROM_DOA[15:0])
	// output reg pmem_ce,
	// output [pmem_width-1:0] pmem_a,
	// input [15:0] pmem_d,

	//, .dmem_ce(RAM_OE)
	, .dmem_we(RAM_WEB)
	, .dmem_a(RAM_ADDRB[10:0])
	, .dmem_di(RAM_DOB[7:0])
	, .dmem_do(RAM_DIB[7:0])
	// output reg dmem_we,
	// output reg [dmem_width-1:0] dmem_a,
	// input [7:0] dmem_di,
	// output reg [7:0] dmem_do,

	, .io_re(IOR_OE)
	, .io_we(IOR_WEB)
	, .io_a(IOR_ADDRB[5:0])
	, .io_do(IOR_DIB[7:0])
	, .io_di(IOR_DOB[7:0])
	// output reg io_re,
	// output reg io_we,
	// output [5:0] io_a,
	// output [7:0] io_do,
	// input [7:0] io_di
	);
// always @(posedge CLK)
//  begin
//    PC <= PC + 1;
//  end

endmodule
